#version 330 core

uniform float progress = 1.0;

uniform sampler2D tex2d;

uniform vec3 color = vec3(1.0, 1.0, 1.0);
uniform int type;

const float smoothing = 1.0 / 16.0;

in vec2 uv;
out vec4 outColor;

float median(float r, float g, float b) {
    return max(min(r, g), min(max(r, g), b));
}

const float pxRange = 2.0;

float screenPxRange() {
    vec2 unitRange = vec2(pxRange) / vec2(textureSize(tex2d, 0));
    vec2 screenTexSize = vec2(1.0) / fwidth(uv);
    return max(0.5 * dot(unitRange, screenTexSize), 1.0);
}

void main() {
    if(type == 0) { // Quads.
        outColor = texture(tex2d, uv);
        outColor.a *= progress;
        outColor.rgb *= color;
    } else if(type == 1) { // Fonts.
        vec3 msd = texture(tex2d, uv).rgb;
        float sd = median(msd.r, msd.g, msd.b);
        float screenPxDistance = screenPxRange() * (sd - 0.5);
        float opacity = clamp(screenPxDistance + 0.5, 0.0, 1.0);
        outColor = vec4(color, opacity);
    }
}